@@ -10,9 +10,7 @@ from aryaxai.client.client import APIClient
 from aryaxai.common.utils import poll_events, pretty_date
 from aryaxai.common.validation import Validate
 from aryaxai.common.xai_uris import (
-    DELETE_SYNTHETIC_MODEL_URI,
-    DELETE_SYNTHETIC_TAG_URI,
-    DOWNLOAD_SYNTHETIC_DATA_URI,
+    AVAILABLE_SYNTHETIC_CUSTOM_SERVERS_URI,
     GENERATE_ANONYMITY_SCORE_URI,
     GENERATE_SYNTHETIC_DATA_URI,
     GET_ANONYMITY_SCORE_URI,
@@ -20,7 +18,7 @@ from aryaxai.common.xai_uris import (
 
 
 class SyntheticDataTag(BaseModel):
-    __api_client: APIClient
+    api_client: APIClient
     project_name: str
     project: Any
 
@@ -39,7 +37,7 @@ class SyntheticDataTag(BaseModel):
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.__api_client = kwargs.get("api_client")
+        self.api_client = kwargs.get("api_client")
 
     def get_model_name(self) -> str:
         """get model type
@@ -76,7 +74,7 @@ class SyntheticModel(BaseModel):
     :return: _description_
     """
 
-    __api_client: APIClient
+    api_client: APIClient
     project_name: str
     project: Any
 
@@ -96,7 +94,7 @@ class SyntheticModel(BaseModel):
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.__api_client = kwargs.get("api_client")
+        self.api_client = kwargs.get("api_client")
 
     def get_model_type(self) -> str:
         """get model type
@@ -160,7 +158,7 @@ class SyntheticModel(BaseModel):
         """
         url = f"{GET_SYNTHETIC_TRAINING_LOGS_URI}?project_name={self.project_name}&model_name={self.model_name}"
 
-        res = self.__api_client.get(url)
+        res = self.api_client.get(url)
 
         if not res['success']:
             raise Exception('Error while getting training logs.')
@@ -168,42 +166,72 @@ class SyntheticModel(BaseModel):
         return res['details']
     '''
 
-    def generate_synthetic_datapoints(self, num_of_datapoints: int):
+    def generate_synthetic_datapoints(
+        self, num_of_datapoints: int, instance_type: Optional[str] = "shared"
+    ):
         """generate given number of synthetic datapoints
 
         :param num_of_datapoints: total datapoints to generate
-        :raises Exception: _description_
+        :param instance_type: type of instance to run training
+            for all available instances check xai.available_synthetic_custom_servers()
+            defaults to shared
         :return: None
         """
+        if instance_type != "shared":
+            available_servers = self.api_client.get(
+                AVAILABLE_SYNTHETIC_CUSTOM_SERVERS_URI
+            )["details"]
+            servers = list(
+                map(lambda instance: instance["instance_name"], available_servers)
+            )
+            Validate.value_against_list("instance_type", instance_type, servers)
+
         payload = {
             "project_name": self.project_name,
             "model_name": self.model_name,
+            "instance_type": instance_type,
             "num_of_datapoints": num_of_datapoints,
         }
 
-        res = self.__api_client.post(GENERATE_SYNTHETIC_DATA_URI, payload)
+        res = self.api_client.post(GENERATE_SYNTHETIC_DATA_URI, payload)
 
         if not res["success"]:
             raise Exception(res["details"])
 
         print("Generating synthetic datapoints...")
         poll_events(
-            self.__api_client,
+            self.api_client,
             self.project_name,
             res["event_id"],
             progress_message="Synthetic Data generation progress",
         )
         print("Synthetic datapoints generated successfully.\n")
 
-    def generate_anonymity_score(self, aux_columns: List[str], control_tag: str):
+    def generate_anonymity_score(
+        self,
+        aux_columns: List[str],
+        control_tag: str,
+        instance_type: Optional[str] = "shared",
+    ):
         """generate anonymity score
 
         :param aux_columns: list of features
         :param control_tag: tag
-        :raises Exception: _description_
-        :raises Exception: _description_
+        :param instance_type: type of instance to run training
+            for all available instances check xai.available_synthetic_custom_servers()
+            defaults to shared
+
         :return: None
         """
+        if instance_type != "shared":
+            available_servers = self.api_client.get(
+                AVAILABLE_SYNTHETIC_CUSTOM_SERVERS_URI
+            )["details"]
+            servers = list(
+                map(lambda instance: instance["instance_name"], available_servers)
+            )
+            Validate.value_against_list("instance_type", instance_type, servers)
+
         if len(aux_columns) < 2:
             raise Exception("aux_columns requires minimum 2 columns.")
 
@@ -222,15 +250,16 @@ class SyntheticModel(BaseModel):
             "control_tag": control_tag,
             "model_name": self.model_name,
             "project_name": self.project_name,
+            "instance_type": instance_type,
         }
 
-        res = self.__api_client.post(GENERATE_ANONYMITY_SCORE_URI, payload)
+        res = self.api_client.post(GENERATE_ANONYMITY_SCORE_URI, payload)
 
         if not res["success"]:
             raise Exception(res["details"])
 
         print("Calculating anonymity score...")
-        poll_events(self.__api_client, self.project_name, res["event_id"])
+        poll_events(self.api_client, self.project_name, res["event_id"])
         print("Anonymity score calculated successfully.\n")
 
     def anonymity_score(self):
@@ -244,7 +273,7 @@ class SyntheticModel(BaseModel):
             "model_name": self.model_name,
         }
 
-        res = self.__api_client.post(GET_ANONYMITY_SCORE_URI, payload)
+        res = self.api_client.post(GET_ANONYMITY_SCORE_URI, payload)
 
         if not res["success"]:
             print(res["details"])
@@ -269,14 +298,14 @@ class SyntheticModel(BaseModel):
 
 
 class SyntheticPrompt(BaseModel):
-    __api_client: APIClient
+    api_client: APIClient
     project: Any
 
     prompt_name: str
     prompt_id: str
     project_name: str
     status: str
-    configuration: List[dict]
+    configuration: List[Any]
     metadata: dict
     created_by: str
     updated_by: str
@@ -285,7 +314,7 @@ class SyntheticPrompt(BaseModel):
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
-        self.__api_client = kwargs.get("api_client")
+        self.api_client = kwargs.get("api_client")
 
     def get_expression(self) -> str:
         """construct prompt expression
@@ -323,7 +352,7 @@ class SyntheticPrompt(BaseModel):
             "update_keys": {}
         }
 
-        res = self.__api_client.post(UPDATE_SYNTHETIC_PROMPT_URI, payload)
+        res = self.api_client.post(UPDATE_SYNTHETIC_PROMPT_URI, payload)
 
         if not res['success']:
             raise Exception(res['details'])
